@@ -58,21 +58,22 @@ class COGReader(PartialReadInterface):
     @property
     def profile(self) -> Dict[str, Any]:
         """Return a rasterio-style image profile"""
-        # TODO: Support nodata value
+        ifd = self.ifds[0]
         return {
             "driver": "GTiff",
-            "width": self.ifds[0].ImageWidth.value,
-            "height": self.ifds[0].ImageHeight.value,
-            "count": self.ifds[0].bands,
-            "dtype": str(self.ifds[0].dtype),
+            "width": ifd.ImageWidth.value,
+            "height": ifd.ImageHeight.value,
+            "count": ifd.bands,
+            "dtype": str(ifd.dtype),
             "transform": self.geotransform(),
-            "blockxsize": self.ifds[0].TileWidth.value,
-            "blockysize": self.ifds[0].TileHeight.value,
-            "compress": self.ifds[0].compression,
-            "interleave": self.ifds[0].interleave,
+            "blockxsize": ifd.TileWidth.value,
+            "blockysize": ifd.TileHeight.value,
+            "compress": ifd.compression,
+            "interleave": ifd.interleave,
             "crs": f"EPSG:{self.epsg}",
+            "nodata": ifd.nodata,
             "tiled": True,
-            "photometric": PHOTOMETRIC[self.ifds[0].PhotometricInterpretation.value],
+            "photometric": PHOTOMETRIC[ifd.PhotometricInterpretation.value],
         }
 
     @property
@@ -106,7 +107,8 @@ class COGReader(PartialReadInterface):
         return {
             'count': self._file_reader._total_requests,
             'byte_count': self._file_reader._total_bytes_requested,
-            'ranges': self._file_reader._requested_ranges
+            'ranges': self._file_reader._requested_ranges,
+            'header_size': self._file_reader._header_size
         }
 
     @property
@@ -114,6 +116,10 @@ class COGReader(PartialReadInterface):
         """Check if the image has an internal mask"""
         return True if self.mask_ifds else False
 
+    @property
+    def nodata(self) -> Optional[int]:
+        return self.ifds[0].nodata
+
     async def _read_header(self) -> None:
         """Internal method to read image header and parse into IFDs and Tags"""
         next_ifd_offset = 1
@@ -161,9 +167,17 @@ class COGReader(PartialReadInterface):
         if z > len(self.ifds):
             raise TileNotFoundError(f"Overview {z} does not exist.")
         ifd = self.ifds[z]
-        idx = (y * ifd.tile_count[0]) + x
-        if idx > len(ifd.TileOffsets):
-            raise TileNotFoundError(f"Tile {x} {y} {z} does not exist")
+        xmax, ymax = ifd.tile_count
+
+        # Return an empty array if tile is outside bounds of image
+        if x < 0 or y < 0 or x >= xmax or y >= ymax:
+            if not config.BOUNDLESS_READ:
+                raise TileNotFoundError(f"Internal tile {z}/{x}/{y} does not exist")
+            tile = np.full(
+                (ifd.bands, ifd.TileHeight.value, ifd.TileWidth.value),
+                fill_value=config.BOUNDLESS_READ_FILL_VALUE
+            )
+            return tile
 
         # Request the tile
         futures.append(
@@ -178,10 +192,15 @@ class COGReader(PartialReadInterface):
             )
 
         tile = await asyncio.gather(*futures)
+
+        # Prioritize internal mask over nodata
         if self.is_masked:
             # Apply mask
             tile[1] = np.invert(np.broadcast_to(tile[1], tile[0].shape))
             return np.ma.masked_array(*tile)
+        # Explicitly check for None because nodata is often 0
+        if ifd.nodata is not None:
+            return np.ma.masked_where(tile[0] == ifd.nodata, tile[0])
         return tile[0]
 
     async def read(self, bounds: Tuple[float, float, float, float], shape: Tuple[int, int]) -> Union[np.ndarray, np.ma.masked_array]:
@@ -201,6 +220,9 @@ class COGReader(PartialReadInterface):
             dtype=ifd.dtype
         )
 
+        if not self._intersect_bounds(bounds, self.bounds):
+            raise TileNotFoundError("Partial read is outside bounds of the image")
+
         # Request those tiles
         if config.HTTP_MERGE_CONSECUTIVE_RANGES:
             img_arr = await self._request_merged_tiles(img_tiles)
