@@ -27,3 +27,15 @@ ENABLE_CACHE: bool = True if os.getenv(
 HTTP_MERGE_CONSECUTIVE_RANGES: bool = True if os.getenv(
     "HTTP_MERGE_CONSECUTIVE_RANGES", "FALSE"
 ).upper() == "TRUE" else False
+
+
+# Determines if internal tiles outside the bounds of the IFD are read. When set to ``TRUE`` (default), if a partial read
+# isn't fully covered by internal tiles, missing tiles will be created using the fill value defined by the
+# ``BOUNDLESS_READ_FILL_VALUE`` config option. When set to ``FALSE``, an exception will be raised instead
+BOUNDLESS_READ: bool = False if os.getenv(
+    "BOUNDLESS_READ", "TRUE"
+) == "FALSE" else True
+
+
+# Determines the fill value used for boundless reads
+BOUNDLESS_READ_FILL_VALUE: int = int(os.getenv("BOUNDLESS_READ_FILL_VALUE", "0"))
