@@ -1,8 +1,6 @@
-import asyncio
-import abc
 from dataclasses import dataclass
 import math
-from typing import Dict, Tuple, Union
+from typing import Dict, Optional, Tuple, Union
 
 import numpy as np
 
@@ -69,14 +67,20 @@ class RequiredTags:
 
 @dataclass
 class OptionalTags:
+    # TIFF standard tags
     NewSubfileType: Tag = None
     Predictor: Tag = None
     JPEGTables: Tag = None
 
+    # GeoTiff
     GeoKeyDirectoryTag: Tag = None
     ModelPixelScaleTag: Tag = None
     ModelTiepointTag: Tag = None
 
+    # GDAL private tags
+    NoData: Tag = None
+
+
 @dataclass
 class ImageIFD(OptionalTags, Compression, RequiredTags, IFD):
 
@@ -102,6 +106,10 @@ class ImageIFD(OptionalTags, Compression, RequiredTags, IFD):
                 SAMPLE_DTYPES[(self.SampleFormat.value[0], self.BitsPerSample.value[0])]
             )
 
+    @property
+    def nodata(self) -> Optional[int]:
+        return int(self.NoData.value[0]) if self.NoData else None
+
     @property
     def interleave(self) -> str:
         """Return the interleave of the IFD"""
@@ -119,8 +127,6 @@ class ImageIFD(OptionalTags, Compression, RequiredTags, IFD):
     async def _get_tile(self, x: int, y: int) -> np.ndarray:
         """Read the requested tile from the IFD"""
         idx = (y * self.tile_count[0]) + x
-        if idx > len(self.TileOffsets):
-            raise TileNotFoundError(f"Tile {x} {y} does not exist")
         offset = self.TileOffsets[idx]
         byte_count = self.TileByteCounts[idx] - 1
         img_bytes = await self._file_reader.range_request(offset, byte_count)
@@ -146,8 +152,6 @@ class MaskIFD(ImageIFD):
     async def _get_tile(self, x: int, y: int) -> np.ndarray:
         """Read the requested tile from the IFD"""
         idx = (y * self.tile_count[0]) + x
-        if idx > len(self.TileOffsets):
-            raise TileNotFoundError(f"Tile {x} {y} does not exist")
         offset = self.TileOffsets[idx]
         byte_count = self.TileByteCounts[idx] - 1
         img_bytes = await self._file_reader.range_request(offset, byte_count)
