@@ -4,13 +4,13 @@ import abc
 from dataclasses import dataclass
 from functools import partial
 import math
-from typing import List, Tuple, Union
+from typing import List, Optional, Tuple, Union
 
 import affine
 import numpy as np
 from skimage.transform import resize
 
-from .filesystems import Filesystem
+from . import config
 from .ifd import ImageIFD, MaskIFD
 from .utils import run_in_background
 
@@ -49,6 +49,12 @@ class PartialReadBase(abc.ABC):
         """Check if the image has an internal mask"""
         ...
 
+    @property
+    @abc.abstractmethod
+    def nodata(self) -> Optional[int]:
+        """Return nodata value or None if there is not a nodata value"""
+        ...
+
     @property
     @abc.abstractmethod
     def overviews(self) -> List[int]:
@@ -72,6 +78,33 @@ class PartialReadBase(abc.ABC):
         """Do a partial read"""
         ...
 
+    @property
+    def _add_mask(self) -> bool:
+        """Determine if a mask needs to be added to the array"""
+        if self.is_masked:
+            return True
+        if self.nodata is not None:
+            return True
+        return False
+
+
+    @staticmethod
+    def _intersect_bounds(
+        read_bounds: Tuple[float, float, float, float],
+        cog_bounds: Tuple[float, float, float, float]
+    ) -> bool:
+        """
+        Determine if a bounding box intersects another bounding box
+
+        https://github.com/cogeotiff/rio-tiler/blob/2.0a11/rio_tiler/utils.py#L254-L283
+        """
+        return (
+            (cog_bounds[0] < read_bounds[2])
+            and (cog_bounds[2] > read_bounds[0])
+            and (cog_bounds[3] > read_bounds[1])
+            and (cog_bounds[1] < read_bounds[3])
+        )
+
     def _get_overview_level(
         self, bounds: Tuple[float, float, float, float], width: int, height: int
     ) -> int:
@@ -171,32 +204,10 @@ class PartialReadBase(abc.ABC):
                 (img_tiles.xmax + 1 - img_tiles.xmin) * img_tiles.tile_width,
             )
         ).astype(img_tiles.dtype)
-        if self.is_masked:
+        if self._add_mask:
             fused = np.ma.masked_array(fused)
         return fused
 
-    @staticmethod
-    def _stitch_image_tile_callback(
-        fut: asyncio.Future,
-        fused_arr: NpArrayType,
-        idx: int,
-        idy: int,
-        tile_width: int,
-        tile_height: int,
-    ) -> None:
-        """Internal asyncio callback used to mosaic each image tile into a larger array (see ``_init_array``)"""
-        img_arr = fut.result()
-        fused_arr[
-            :,
-            idy * tile_height : (idy + 1) * tile_height,
-            idx * tile_width : (idx + 1) * tile_width,
-        ] = img_arr
-        if np.ma.is_masked(img_arr):
-            fused_arr.mask[
-                :,
-                idy * tile_height : (idy + 1) * tile_height,
-                idx * tile_width : (idx + 1) * tile_width,
-            ] = img_arr.mask
 
     @staticmethod
     def _stitch_image_tile(
@@ -220,6 +231,21 @@ class PartialReadBase(abc.ABC):
                 idx * tile_width : (idx + 1) * tile_width,
             ] = arr.mask
 
+
+    async def _get_and_stitch_tile(
+        self,
+        xtile: int,
+        ytile: int,
+        idx: int,
+        idy: int,
+        img_tiles: TileMetadata,
+        fused_arr: NpArrayType
+    ) -> None:
+        """Asynchronously request an internal tile and stitch into an array"""
+        tile = await self.get_tile(xtile, ytile, img_tiles.ovr_level)
+        self._stitch_image_tile(tile, fused_arr, idx, idy, img_tiles.tile_width, img_tiles.tile_height)
+
+
     async def _request_tiles(self, img_tiles: TileMetadata) -> NpArrayType:
         """Concurrently request the image tiles and mosaic into a larger array"""
         img_arr = self._init_array(img_tiles)
@@ -227,17 +253,7 @@ class PartialReadBase(abc.ABC):
         for idx, xtile in enumerate(range(img_tiles.xmin, img_tiles.xmax + 1)):
             for idy, ytile in enumerate(range(img_tiles.ymin, img_tiles.ymax + 1)):
                 get_tile_task = asyncio.create_task(
-                    self.get_tile(xtile, ytile, img_tiles.ovr_level)
-                )
-                get_tile_task.add_done_callback(
-                    partial(
-                        self._stitch_image_tile_callback,
-                        fused_arr=img_arr,
-                        idx=idx,
-                        idy=idy,
-                        tile_width=img_tiles.tile_width,
-                        tile_height=img_tiles.tile_height,
-                    )
+                    self._get_and_stitch_tile(xtile, ytile, idx, idy, img_tiles, img_arr)
                 )
                 tile_tasks.append(get_tile_task)
         await asyncio.gather(*tile_tasks)
@@ -261,7 +277,7 @@ class PartialReadBase(abc.ABC):
             preserve_range=True,
             anti_aliasing=True,
         ).astype(img_tiles.dtype)
-        if self.is_masked:
+        if self._add_mask:
             resized_mask = resize(
                 clipped.mask,
                 output_shape=(img_tiles.bands, out_shape[0], out_shape[1]),
@@ -320,7 +336,7 @@ class PartialReadInterface(PartialReadBase):
         )
         futures.append(tile_task)
 
-        if self.is_masked:
+        if self._add_mask:
             # Request mask data
             mask_ifd = self.mask_ifds[img_tiles.ovr_level]
             mask_offset = mask_ifd.TileOffsets[min(tile_indices)]
@@ -340,7 +356,7 @@ class PartialReadInterface(PartialReadBase):
             tile_bytes = self._extract_tile(ifd, response[0], tile_idx, offset)
             # Decompress the tile
             decoded = await run_in_background(ifd._decompress, tile_bytes)
-            if self.is_masked:
+            if self._add_mask:
                 # Extract mask
                 mask_ifd = self.mask_ifds[img_tiles.ovr_level]
                 mask_bytes = self._extract_tile(
